public inbox for passt-dev@passt.top
 help / color / mirror / code / Atom feed
From: David Gibson <david@gibson.dropbear.id.au>
To: Stefano Brivio <sbrivio@redhat.com>, passt-dev@passt.top
Cc: David Gibson <david@gibson.dropbear.id.au>
Subject: [PATCH v3 10/16] udp: Update UDP "connection" timestamps in both directions
Date: Fri, 25 Nov 2022 18:29:10 +1100	[thread overview]
Message-ID: <20221125072916.3060938-11-david@gibson.dropbear.id.au> (raw)
In-Reply-To: <20221125072916.3060938-1-david@gibson.dropbear.id.au>

A UDP pseudo-connection between port A in the init namespace and port B in
the pasta guest namespace involves two sockets: udp_splice_init[v6][B]
and udp_splice_ns[v6][A].  The socket which originated this "connection"
will be permanent but the other one will be closed on a timeout.

When we get a packet from the originating socket, we update the timeout on
the other socket, but we don't do the same when we get a reply packet from
the other socket.  However any activity on the "connection" probably
indicates that it's still in use.  Without this we could incorrectly time
out a "connection" if it's using a protocol which involves a single
initiating packet, but which then gets continuing replies from the target.

Correct this by updating the timeout on both sockets for a packet in either
direction.  This also updates the timestamps for the permanent originating
sockets which is unnecessary, but harmless.

Signed-off-by: David Gibson <david@gibson.dropbear.id.au>
---
 udp.c | 19 +++++++++++++++++--
 1 file changed, 17 insertions(+), 2 deletions(-)

diff --git a/udp.c b/udp.c
index 97453e7..d835a89 100644
--- a/udp.c
+++ b/udp.c
@@ -55,12 +55,15 @@
  *       - bind in namespace to 127.0.0.1:5000
  *       - add to epoll with reference: index = 5000, splice = 1, orig = 0,
  *         ns = 1
  *     - update udp_splice_ns[V4][5000].ts with current time
  *
  *   - reverse direction: 127.0.0.1:80 -> 127.0.0.1:5000 in namespace socket s,
  *     having epoll reference: index = 5000, splice = 1, orig = 0, ns = 1
  *     - if udp_splice_init[V4][80].sock:
  *       - send to udp_splice_init[V4][80].sock, with destination port 5000
+ *       - update udp_splice_init[V4][80].ts and udp_splice_ns[V4][5000].ts with
+ *         current time
  *     - otherwise, discard
  *
  * - from namespace to init:
@@ -75,12 +78,15 @@
  *       - bind in init to 127.0.0.1:2000
  *       - add to epoll with reference: index = 2000, splice = 1, orig = 0,
  *         ns = 0
  *     - update udp_splice_init[V4][2000].ts with current time
  *
  *   - reverse direction: 127.0.0.1:22 -> 127.0.0.1:2000 in init from socket s,
  *     having epoll reference: index = 2000, splice = 1, orig = 0, ns = 0
  *   - if udp_splice_ns[V4][22].sock:
  *     - send to udp_splice_ns[V4][22].sock, with destination port 2000
+ *     - update udp_splice_ns[V4][22].ts and udp_splice_init[V4][2000].ts with
+ *       current time
  *   - otherwise, discard
  */
 
@@ -539,12 +545,16 @@ static void udp_sock_handler_splice(const struct ctx *c, union epoll_ref ref,
 				return;
 		}
 
+		udp_splice_init[v6][dst].ts = now->tv_sec;
 		udp_splice_ns[v6][src].ts = now->tv_sec;
 	} else if (!ref.r.p.udp.udp.orig && ref.r.p.udp.udp.ns) {
 		src += c->udp.fwd_in.rdelta[src];
 
 		if (!(s = udp_splice_init[v6][src].sock))
 			return;
+
+		udp_splice_ns[v6][dst].ts = now->tv_sec;
+		udp_splice_init[v6][src].ts = now->tv_sec;
 	} else if (ref.r.p.udp.udp.orig && ref.r.p.udp.udp.ns) {
 		src += c->udp.fwd_in.rdelta[src];
 
@@ -553,12 +563,17 @@ static void udp_sock_handler_splice(const struct ctx *c, union epoll_ref ref,
 			if (s < 0)
 				return;
 		}
+
+		udp_splice_ns[v6][dst].ts = now->tv_sec;
 		udp_splice_init[v6][src].ts = now->tv_sec;
 	} else if (!ref.r.p.udp.udp.orig && !ref.r.p.udp.udp.ns) {
 		src += c->udp.fwd_out.rdelta[src];
 
 		if (!(s = udp_splice_ns[v6][src].sock))
 			return;
+
+		udp_splice_init[v6][dst].ts = now->tv_sec;
+		udp_splice_ns[v6][src].ts = now->tv_sec;
 	} else {
 		return;
 	}
@@ -55,12 +55,15 @@
  *       - add to epoll with reference: index = 5000, splice = 1, orig = 0,
  *         ns = 1
- *     - update udp_splice_ns[V4][5000].ts with current time
+ *     - update udp_splice_init[V4][80].ts and udp_splice_ns[V4][5000].ts with
+ *       current time
  *
  *   - reverse direction: 127.0.0.1:80 -> 127.0.0.1:5000 in namespace socket s,
@@ -75,12 +78,15 @@
  *       - add to epoll with reference: index = 2000, splice = 1, orig = 0,
  *         ns = 0
- *     - update udp_splice_init[V4][2000].ts with current time
+ *     - update udp_splice_ns[V4][22].ts and udp_splice_init[V4][2000].ts with
+ *       current time
  *
  *   - reverse direction: 127.0.0.1:22 -> 127.0.0.1:2000 in init from socket s,
-- 
@@ -55,12 +55,15 @@
  *       - bind in namespace to 127.0.0.1:5000
  *       - add to epoll with reference: index = 5000, splice = 1, orig = 0,
  *         ns = 1
- *     - update udp_splice_ns[V4][5000].ts with current time
+ *     - update udp_splice_init[V4][80].ts and udp_splice_ns[V4][5000].ts with
+ *       current time
  *
  *   - reverse direction: 127.0.0.1:80 -> 127.0.0.1:5000 in namespace socket s,
  *     having epoll reference: index = 5000, splice = 1, orig = 0, ns = 1
  *     - if udp_splice_init[V4][80].sock:
  *       - send to udp_splice_init[V4][80].sock, with destination port 5000
+ *       - update udp_splice_init[V4][80].ts and udp_splice_ns[V4][5000].ts with
+ *         current time
  *     - otherwise, discard
  *
  * - from namespace to init:
@@ -75,12 +78,15 @@
  *       - bind in init to 127.0.0.1:2000
  *       - add to epoll with reference: index = 2000, splice = 1, orig = 0,
  *         ns = 0
- *     - update udp_splice_init[V4][2000].ts with current time
+ *     - update udp_splice_ns[V4][22].ts and udp_splice_init[V4][2000].ts with
+ *       current time
  *
  *   - reverse direction: 127.0.0.1:22 -> 127.0.0.1:2000 in init from socket s,
  *     having epoll reference: index = 2000, splice = 1, orig = 0, ns = 0
  *   - if udp_splice_ns[V4][22].sock:
  *     - send to udp_splice_ns[V4][22].sock, with destination port 2000
+ *     - update udp_splice_ns[V4][22].ts and udp_splice_init[V4][2000].ts with
+ *       current time
  *   - otherwise, discard
  */
 
@@ -539,12 +545,16 @@ static void udp_sock_handler_splice(const struct ctx *c, union epoll_ref ref,
 				return;
 		}
 
+		udp_splice_init[v6][dst].ts = now->tv_sec;
 		udp_splice_ns[v6][src].ts = now->tv_sec;
 	} else if (!ref.r.p.udp.udp.orig && ref.r.p.udp.udp.ns) {
 		src += c->udp.fwd_in.rdelta[src];
 
 		if (!(s = udp_splice_init[v6][src].sock))
 			return;
+
+		udp_splice_ns[v6][dst].ts = now->tv_sec;
+		udp_splice_init[v6][src].ts = now->tv_sec;
 	} else if (ref.r.p.udp.udp.orig && ref.r.p.udp.udp.ns) {
 		src += c->udp.fwd_in.rdelta[src];
 
@@ -553,12 +563,17 @@ static void udp_sock_handler_splice(const struct ctx *c, union epoll_ref ref,
 			if (s < 0)
 				return;
 		}
+
+		udp_splice_ns[v6][dst].ts = now->tv_sec;
 		udp_splice_init[v6][src].ts = now->tv_sec;
 	} else if (!ref.r.p.udp.udp.orig && !ref.r.p.udp.udp.ns) {
 		src += c->udp.fwd_out.rdelta[src];
 
 		if (!(s = udp_splice_ns[v6][src].sock))
 			return;
+
+		udp_splice_init[v6][dst].ts = now->tv_sec;
+		udp_splice_ns[v6][src].ts = now->tv_sec;
 	} else {
 		return;
 	}
-- 
2.38.1


  parent reply	other threads:[~2022-11-25  7:29 UTC|newest]

Thread overview: 18+ messages / expand[flat|nested]  mbox.gz  Atom feed  top
2022-11-25  7:29 [PATCH v3 00/16] Simplify and correct handling of "spliced" UDP forwarding David Gibson
2022-11-25  7:29 ` [PATCH v3 01/16] udp: Also bind() connected ports for "splice" forwarding David Gibson
2022-11-25  7:29 ` [PATCH v3 02/16] udp: Separate tracking of inbound and outbound packet flows David Gibson
2022-11-25  7:29 ` [PATCH v3 03/16] udp: Always use sendto() rather than send() for forwarding spliced packets David Gibson
2022-11-25  7:29 ` [PATCH v3 04/16] udp: Don't connect "forward" sockets for spliced flows David Gibson
2022-11-25  7:29 ` [PATCH v3 05/16] udp: Remove the @bound field from union udp_epoll_ref David Gibson
2022-11-25  7:29 ` [PATCH v3 06/16] udp: Split splice field in udp_epoll_ref into (mostly) independent bits David Gibson
2022-11-25  7:29 ` [PATCH v3 07/16] udp: Don't create double sockets for -U port David Gibson
2022-11-25  7:29 ` [PATCH v3 08/16] udp: Re-use fixed bound sockets for packet forwarding when possible David Gibson
2022-11-25  7:29 ` [PATCH v3 09/16] udp: Don't explicitly track originating socket for spliced "connections" David Gibson
2022-11-25  7:29 ` David Gibson [this message]
2022-11-25  7:29 ` [PATCH v3 11/16] udp: Simplify udp_sock_handler_splice David Gibson
2022-11-25  7:29 ` [PATCH v3 12/16] udp: Make UDP_SPLICE_FRAMES and UDP_TAP_FRAMES_MEM the same thing David Gibson
2022-11-25  7:29 ` [PATCH v3 13/16] udp: Add helper to extract port from a sockaddr_in or sockaddr_in6 David Gibson
2022-11-25  7:29 ` [PATCH v3 14/16] udp: Unify buffers for tap and splice paths David Gibson
2022-11-25  7:29 ` [PATCH v3 15/16] udp: Split send half of udp_sock_handler_splice() from the receive half David Gibson
2022-11-25  7:29 ` [PATCH v3 16/16] udp: Correct splice forwarding when receiving from multiple sources David Gibson
2022-11-29  5:59   ` David Gibson

Reply instructions:

You may reply publicly to this message via plain-text email
using any one of the following methods:

* Save the following mbox file, import it into your mail client,
  and reply-to-all from there: mbox

  Avoid top-posting and favor interleaved quoting:
  https://en.wikipedia.org/wiki/Posting_style#Interleaved_style

* Reply using the --to, --cc, and --in-reply-to
  switches of git-send-email(1):

  git send-email \
    --in-reply-to=20221125072916.3060938-11-david@gibson.dropbear.id.au \
    --to=david@gibson.dropbear.id.au \
    --cc=passt-dev@passt.top \
    --cc=sbrivio@redhat.com \
    /path/to/YOUR_REPLY

  https://kernel.org/pub/software/scm/git/docs/git-send-email.html

* If your mail client supports setting the In-Reply-To header
  via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line before the message body.
Code repositories for project(s) associated with this public inbox

	https://passt.top/passt

This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox;
as well as URLs for IMAP folder(s).